"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deepExtend = require("deep-extend");
const fs = require("fs");
const path = require("path");
const writePkg = require("write-pkg");
const conf_1 = require("../services/conf");
const docker_1 = require("../services/docker");
const render_1 = require("./render");
const { resolve, join } = path;
const { RdTypes, rdcConfName, dockerWorkDirRoot, } = conf_1.default;
class Sync {
    get appConf() {
        return require(resolve(conf_1.default.cwd, conf_1.default.appConfName));
    }
    get from() {
        if (conf_1.default.isApp) {
            const { container } = conf_1.default.getAppConf();
            return container.name;
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConfPath = resolve(conf_1.default.cwd, rdcConfName);
            const rdcConf = require(rdcConfPath);
            return rdcConf.nodeVersion || 'node:latest';
        }
    }
    get ports() {
        if (conf_1.default.isApp) {
            const { docker } = conf_1.default.getAppConf();
            return docker.ports || [];
        }
        if (conf_1.default.rdType === RdTypes.Container) {
            const rdcConf = require(join(conf_1.default.cwd, conf_1.default.rdcConfName));
            return rdcConf.docker.ports || [];
        }
    }
    /**
     * sync all
     * used for local dev env, resync is needed when container updates
     * 1. generate Dockerfile to local .cache
     * 2. generate docker-compose to local .cache
     *
     * if rdType is app
     * need to gen extra staged files
     */
    async start({ watch, cmd }) {
        let volumns = [];
        if (conf_1.default.isApp) {
            await this.genAppStagedFiles();
            const rdcConfPath = resolve(conf_1.default.localCacheDir, rdcConfName);
            const rdcConf = require(rdcConfPath);
            volumns = rdcConf.exportFiles;
        }
        await docker_1.default.genDockerFile(conf_1.default.dockerWorkDirRoot, this.from, conf_1.default.localCacheDir, conf_1.default.isApp);
        // @TODO: docker-compose merge
        // needed in order to let container developer write multiple services like mongo
        await docker_1.default.genDockerCompose(conf_1.default.dockerWorkDirRoot, cmd, this.ports, watch, `dev-${conf_1.default.tag}`, conf_1.default.localCacheDir, conf_1.default.isApp, cmd === 'build', volumns);
    }
    /**
     * 1. copy rdc/package.json to .cache/package.json
     * 2. copy rdc/rdc.config.js to .cache/rdc.config.js
     * 3. add suite to package.json
     * 4. merge app optional pkg.json & rdc package.json
     */
    async genAppStagedFiles(createRdc = null) {
        const { cwd, rdcConfName, dockerRdcDir, localCacheDir, } = conf_1.default;
        // only create application will pass rdc param
        const image = createRdc || this.appConf.container.name;
        await docker_1.default.copy(image, [{
                from: resolve(dockerRdcDir, 'package.json'),
                to: localCacheDir,
            }, {
                from: resolve(dockerRdcDir, rdcConfName),
                to: localCacheDir,
            }]);
        if (!createRdc) {
            await this.mergePkgJson(resolve(cwd, 'package.json'), resolve(cwd, localCacheDir, 'package.json'), resolve(localCacheDir));
        }
        await this.genDevcontainer(createRdc);
    }
    async genDevcontainer(createRdc) {
        const rdcConfPath = resolve(conf_1.default.localCacheDir, rdcConfName);
        const rdcConf = require(rdcConfPath);
        const map = {
            devcontainer: 'devcontainer.json',
            'docker-compose': 'docker-compose.yml',
            Dockerfile: 'Dockerfile',
        };
        const from = createRdc || this.from;
        rdcConf.docker = rdcConf.docker || {};
        const ports = createRdc ? rdcConf.docker.ports || [] : this.ports;
        const exportFiles = rdcConf.exportFiles || [];
        const extensions = rdcConf.extensions || [];
        for (let [key, value] of Object.entries(map)) {
            await render_1.default.renderTo(`devcontainer/${key}`, {
                tag: conf_1.default.tag,
                workDir: dockerWorkDirRoot,
                extensions: JSON.stringify(extensions),
                from,
                ports,
                exportFiles,
            }, resolve(conf_1.default.cwd, '.devcontainer', value), {
                overwrite: true,
            });
        }
    }
    async mergePkgJson(appPkgPath, rdcPkgPath, destPath) {
        let pkgJson = require(rdcPkgPath);
        this.appConf.suites.map(item => {
            pkgJson.dependencies = pkgJson.dependencies || {};
            pkgJson.dependencies[item.name] = item.version;
        });
        if (fs.existsSync(appPkgPath) &&
            fs.statSync(appPkgPath).isFile()) {
            const appPkgJson = require(appPkgPath);
            deepExtend(pkgJson, appPkgJson);
        }
        await writePkg(destPath, pkgJson);
    }
}
exports.default = new Sync();
